--[[
	Finds any bones named "Dangler", and adjusts the angle of these bones to always point downwards.
]]

-- **************************************************
-- Recurring values
-- **************************************************

LM_HangingBones = {}

LM_HangingBones.dialogUsed = false
LM_HangingBones.angle = math.rad(270)

-- **************************************************
-- Dangler dialog
-- **************************************************

local LM_HangingBonesDialog = {}

function LM_HangingBonesDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Hanging Bones", LM_HangingBonesDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Hang direction"))
		d.angle = LM.GUI.AngleWidget()
		l:AddChild(d.angle)
	l:Pop()

	return d
end

function LM_HangingBonesDialog:UpdateWidgets()
	self.angle:SetValue(LM_HangingBones.angle)
end

function LM_HangingBonesDialog:OnOK()
	LM_HangingBones.angle = self.angle:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LayerScript(moho)
	if (not LM_HangingBones.dialogUsed) then -- if the dialog has not been invoked before, do it now
		local dlog = LM_HangingBonesDialog:new(moho)
		if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
			LM_HangingBones.dialogUsed = true -- the user cannot really cancel at this point, since the script has no way to remove itself from a layer
			return
		end
		LM_HangingBones.dialogUsed = true -- the dialog stage is done, do not invoke it again
	end

	if (moho.frame == 0) then
		return
	end
	local skel = moho:Skeleton()
	if (skel == nil) then
		print("No skeleton found in layer:", moho.layer:Name())
		return
	end
	local origin = LM.Vector2:new_local()
	local tip = LM.Vector2:new_local()
	local matrix = LM.Matrix:new_local()
	moho.layer:GetFullTransform(moho.frame, matrix, nil)
	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone:Name() == "Dangler") then
			origin:Set(0, 0)
			bone.fMovedMatrix:Transform(origin)
			matrix:Transform(origin)
			tip:Set(0, 0.1)
			bone.fMovedMatrix:Transform(tip)
			matrix:Transform(tip)
			local angle = math.atan2(tip.y - origin.y, tip.x - origin.x)
			bone.fAngle = (bone.fAngle - angle) + (LM_HangingBones.angle - math.rad(270))
		end
	end
	skel:UpdateBoneMatrix()
end
